@@ -2,6 +2,15 @@ from unittest import TestCase
 
 import routes
 
+def raises(exc, func, *args, **kw):
+    try:
+        func(*args, **kw)
+    except exc:
+        pass
+    else:
+        tup = func.__name__, e.__name__
+        raise AssertionError("%s() did not raise %s" % tup)
+
 def test_environ():
     return {
         'HTTP_HOST': 'bob.local:5000',
@@ -37,12 +46,3 @@ class WebHelpersTestCase(TestCase):
             'wsgi.url_scheme': 'http'
             }
 
-    def setUp(self):
-        map = routes.Mapper()
-        map.connect('test')
-        map.connect(':controller/:action/:id')
-
-        self.routes_config = routes.request_config()
-        self.routes_config.mapper = map
-        self.routes_config.environ = self.test_environ()
-        assert self.routes_config.mapper_dict
