@@ -6,18 +6,24 @@ BeautifulSoup and HTMLTidy handle this well.
 
 import re
 import urllib
+import warnings
 
 from webhelpers.html import HTML, literal, lit_sub, escape
-from webhelpers.html.tags import convert_boolean_attrs
+import webhelpers.html.tags as tags
 
 __all__ = [
-    'button_to', 
-    'mail_to',
-    'highlight', 
-    'strip_links',
     'auto_link', 
+    'button_to', 
+    'js_obfuscate',
+    'highlight', 
+    'mail_to',
+    'strip_links',
+    'strip_tags',
     ]
 
+tag_re = re.compile(r'<.*?>', re.S)
+br_re = re.compile(r'<br.*?>', re.I|re.S)
+comment_re = re.compile(r'<!--|-->')
 
 AUTO_LINK_RE = re.compile(r"""
                         (                          # leading text
@@ -34,14 +40,14 @@ AUTO_LINK_RE = re.compile(r"""
                           (?:\.[-\w]+)*            # remaining subdomains or domain
                           (?::\d+)?                # port
                           (?:/(?:(?:[~\w\+%-]|(?:[,.;:][^\s$]))+)?)* # path
-                          (?:\?[\w\+%&=.;-]+)?     # query string
+                          (?:\?[\w\+\/%&=.;-]+)?     # query string
                           (?:\#[\w\-]*)?           # trailing anchor
                         )
-                        ([\.,"'?!;:]|\s|<|$)       # trailing text
+                        ([\.,"'?!;:]|\s|<|\]|$)       # trailing text
                            """, re.X)
 
 
-def button_to(name, url='', **html_options):
+def button_to(name, url='', **html_attrs):
     """Generate a form containing a sole button that submits to
     ``url``. 
     
@@ -49,12 +55,12 @@ def button_to(name, url='', **html_options):
     the safe HTTP GET semantics implied by using a hypertext link.
     
     The parameters are the same as for ``link_to``.  Any 
-    ``html_options`` that you pass will be applied to the inner
+    ``html_attrs`` that you pass will be applied to the inner
     ``input`` element. In particular, pass
     
         disabled = True/False
     
-    as part of ``html_options`` to control whether the button is
+    as part of ``html_attrs`` to control whether the button is
     disabled.  The generated form element is given the class
     'button-to', to which you can attach CSS styles for display
     purposes.
@@ -108,11 +114,11 @@ def button_to(name, url='', **html_options):
         (Bottom line: Always validate your HTML before going public.)
     
     """
-    if html_options:
-        convert_boolean_attrs(html_options, ['disabled'])
+    if html_attrs:
+        tags.convert_boolean_attrs(html_attrs, ['disabled'])
     
     method_tag = ''
-    method = html_options.pop('method', '')
+    method = html_attrs.pop('method', '')
     if method.upper() in ['PUT', 'DELETE']:
         method_tag = HTML.input(
             type='hidden', id='_method', name_='_method', value=method)
@@ -121,27 +127,42 @@ def button_to(name, url='', **html_options):
     
     url, name = url, name or url
     
-    submit_type = html_options.get('type')
-    img_source = html_options.get('src')
+    submit_type = html_attrs.get('type')
+    img_source = html_attrs.get('src')
     if submit_type == 'image' and img_source:
-        html_options["value"] = name
-        html_options.setdefault("alt", name)
+        html_attrs["value"] = name
+        html_attrs.setdefault("alt", name)
     else:
-        html_options["type"] = "submit"
-        html_options["value"] = name
+        html_attrs["type"] = "submit"
+        html_attrs["value"] = name
     
     return HTML.form(method=form_method, action=url, class_="button-to",
-                     c=[HTML.div(method_tag, HTML.input(**html_options))])
+                     c=[HTML.div(method_tag, HTML.input(**html_attrs))])
+
+def js_obfuscate(content):
+    """Obfuscate data in a Javascript tag.
+    
+    Example::
+        
+        >>> js_obfuscate("<input type='hidden' name='check' value='valid' />")
+        literal(u'<script type="text/javascript">\\n//<![CDATA[\\neval(unescape(\\'%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%69%6e%70%75%74%20%74%79%70%65%3d%27%68%69%64%64%65%6e%27%20%6e%61%6d%65%3d%27%63%68%65%63%6b%27%20%76%61%6c%75%65%3d%27%76%61%6c%69%64%27%20%2f%3e%27%29%3b\\'))\\n//]]>\\n</script>')
+        
+    """
+    doc_write = "document.write('%s');" % content
+    obfuscated = ''.join(['%%%x' % ord(x) for x in doc_write])
+    complete = "eval(unescape('%s'))" % obfuscated
+    cdata = HTML.cdata("\n", complete, "\n//")
+    return HTML.script("\n//", cdata, "\n", type="text/javascript")
 
 
 def mail_to(email_address, name=None, cc=None, bcc=None, subject=None, 
-    body=None, replace_at=None, replace_dot=None, encode=None, **html_options):
+    body=None, replace_at=None, replace_dot=None, encode=None, **html_attrs):
     """Create a link tag for starting an email to the specified 
     ``email_address``.
     
     This ``email_address`` is also used as the name of the link unless
     ``name`` is specified. Additional HTML options, such as class or
-    id, can be passed in the ``html_options`` hash.
+    id, can be passed in the ``html_attrs`` hash.
     
     You can also make it difficult for spiders to harvest email address
     by obfuscating them.
@@ -203,9 +224,9 @@ def mail_to(email_address, name=None, cc=None, bcc=None, subject=None,
     url = HTML.literal(protocol + email_address)
     if options_query:
         url += HTML.literal('?') + options_query
-    html_options['href'] = url
+    html_attrs['href'] = url
 
-    tag = HTML.a(name or email_address_obfuscated, **html_options)
+    tag = HTML.a(name or email_address_obfuscated, **html_attrs)
 
     if encode == 'javascript':
         tmp = "document.write('%s');" % tag
@@ -217,45 +238,92 @@ def mail_to(email_address, name=None, cc=None, bcc=None, subject=None,
         return tag
 
 
-def highlight(text, phrase, 
-              highlighter='<strong class="highlight">\\1</strong>'):
-    """Highlight the ``phrase`` where it is found in the ``text``.
-    
-    The highlighted phrase will be surrounded by the highlighter, 
-    by default::
-    
-        <strong class="highlight">I'm a highlight phrase</strong>
-    
-    ``highlighter``
-        Defines the highlighting phrase. This argument should be a 
-        single-quoted string with ``\\1`` where the phrase is supposed 
-        to be inserted.
-        
-    Note: The ``phrase`` is sanitized to include only letters, digits, 
-    and spaces before use.
 
-    Example::
+def highlight(text, phrase, highlighter=None, case_sensitive=False, 
+    class_="highlight", **attrs):
+    """Highlight all occurrences of ``phrase`` in ``text``.
 
-        >>> highlight('You searched for: Pylons', 'Pylons')
-        'You searched for: <strong class="highlight">Pylons</strong>'
-        
+    This inserts "<strong class="highlight">...</strong>" around every
+    occurrence.
+
+    Arguments:
+    
+    ``text``: 
+        The full text.
+    
+    ``phrase``: 
+        A phrase to find in the text. This may be a string, a list of strings, 
+        or a compiled regular expression. If a string, it's regex-escaped and
+        compiled. If a list, all of the strings will be highlighted.  This is
+        done by regex-escaping all elements and then joining them using the
+        regex "|" token.
+
+    ``highlighter``:
+        Deprecated.  A replacement expression for the regex substitution.
+        This was deprecated because it bypasses the HTML builder and creates
+        tags via string mangling.  The previous default was '<strong
+        class="highlight">\\1</strong>', which mimics the normal behavior of
+        this function.  ``phrase`` must be a string if ``highlighter`` is
+        specified.  Overrides ``class_`` and ``attrs_`` arguments.
+
+    ``case_sensitive``:
+        If false (default), the phrases are searched in a case-insensitive
+        manner. No effect if ``phrase`` is a regex object.
+
+    ``class_``:
+        CSS class for the <strong> tag.
+
+    ``**attrs``:
+        Additional HTML attributes for the <strong> tag.
+
+    Changed in WebHelpers 1.0b2: new implementation using HTML builder.
+    Allow ``phrase`` to be list or regex.  Deprecate ``highlighter`` and
+    change its default value to None. Add ``case_sensitive``, ``class_``,
+    and ``**attrs`` arguments.
     """
     if not phrase or not text:
         return text
-    highlight_re = re.compile('(%s)' % re.escape(phrase), re.I)
-    if hasattr(text, '__html__'):
-        return literal(highlight_re.sub(highlighter, text))
+    text = escape(text)
+    if case_sensitive:
+        flags = 0   # No flags.
     else:
-        return highlight_re.sub(highlighter, text)
+        flags = re.IGNORECASE
+    if highlighter:
+        return _legacy_highlight(text, phrase, highlighter, flags)
+    if isinstance(phrase, basestring):
+        pat = re.escape(phrase)
+        rx = re.compile(pat, flags)
+    elif isinstance(phrase, (list, tuple)):
+        parts = [re.escape(x) for x in phrase]
+        pat = "|".join(parts)
+        rx = re.compile(pat, flags)
+    else:
+        rx = phrase
+    def repl(m):
+        return HTML.strong(m.group(), class_=class_, **attrs)
+    return lit_sub(rx, repl, text)
 
 
-def auto_link(text, link="all", **href_options):
+def _legacy_highlight(text, phrase, highlighter, flags):
+    """WebHelpers 0.6 style highlight with deprecated ``highlighter arg."""
+    warnings.warn("the ``highlighter`` argument is deprecated",
+        DeprecationWarning)
+    pat = "(%s)" % re.escape(phrase)
+    rx = re.compile(pat, flags)
+    highlighter = literal(highlighter)
+    return lit_sub(rx, highlighter, text)
+    
+
+def auto_link(text, link="all", **href_attrs):
     """
     Turn all urls and email addresses into clickable links.
     
     ``link``
         Used to determine what to link. Options are "all", 
         "email_addresses", or "urls"
+
+    ``href_attrs``
+        Additional attributes for generated <a> tags.
     
     Example::
     
@@ -264,15 +332,16 @@ def auto_link(text, link="all", **href_options):
         
     """
     if not text:
-        return u""
+        return literal(u"")
+    text = escape(text)
     if link == "all":
-        return _auto_link_urls(_auto_link_email_addresses(text), **href_options)
+        return _auto_link_urls(_auto_link_email_addresses(text), **href_attrs)
     elif link == "email_addresses":
         return _auto_link_email_addresses(text)
     else:
-        return _auto_link_urls(text, **href_options)
+        return _auto_link_urls(text, **href_attrs)
 
-def _auto_link_urls(text, **href_options):
+def _auto_link_urls(text, **href_attrs):
     def handle_match(matchobj):
         all = matchobj.group()
         before, prefix, link, after = matchobj.group(1, 2, 3, 4)
@@ -281,14 +350,14 @@ def _auto_link_urls(text, **href_options):
         text = literal(prefix + link)
         if prefix == "www.":
             prefix = "http://www."
-        a_options = dict(href_options)
+        a_options = dict(href_attrs)
         a_options['href'] = literal(prefix + link)
         return literal(before) + HTML.a(text, **a_options) + literal(after)
     return literal(re.sub(AUTO_LINK_RE, handle_match, text))
 
 def _auto_link_email_addresses(text):
     return lit_sub(r'([\w\.!#\$%\-+.]+@[A-Za-z0-9\-]+(\.[A-Za-z0-9\-]+)+)',
-                   r'<a href="mailto:\1">\1</a>', text)
+                   literal(r'<a href="mailto:\1">\1</a>'), text)
 
 def strip_links(text):
     """
@@ -306,3 +375,27 @@ def strip_links(text):
         lit = lambda x: x
     strip_re = re.compile(r'<a\b.*?>(.*?)<\/a>', re.I | re.M)
     return lit(strip_re.sub(r'\1', text))
+
+def strip_tags(text):
+    """Delete any HTML tags in the text, leaving their contents intact.
+    Convert newlines to spaces, and <br /> to newlines.
+
+    Example::
+        >>> strip_tags('Text <em>emphasis</em> <script>Javascript</script>.')
+        'Text emphasis Javascript.'
+        >>> strip_tags('Ordinary <!-- COMMENT! --> text.')
+        'Ordinary  COMMENT!  text.'
+        >>> strip_tags('Line\\n1<br />Line 2')
+        'Line 1\\nLine 2'
+
+    Implementation copied from ``WebOb``.
+
+    ``webhelpers.html.converters`` contains more sophisticated versions of 
+    this.
+    """
+    text = text.replace('\n', ' ')
+    text = text.replace('\r', '')
+    text = br_re.sub('\n', text)
+    text = comment_re.sub('', text)
+    text = tag_re.sub('', text)
+    return text
