@@ -1,7 +1,10 @@
-"""Number formatting and numeric helpers"""
+"""Number formatting, numeric helpers, and numeric statistics."""
 
+import math
 import re
 
+#### Calculations ####
+
 def percent_of(part, whole):
     """What percent of ``whole`` is ``part``?
 
@@ -13,10 +16,10 @@ def percent_of(part, whole):
     # Use float to force true division.
     return float(part * 100) / whole
 
+#### Statistics ####
+
 def mean(r):
-    """Return the mean of a sequence of numbers.
-    
-    The mean is the average of all the numbers.
+    """Return the mean (i.e., average) of a sequence of numbers.
 
     >>> mean([5, 10])
     7.5
@@ -51,15 +54,6 @@ def median(r):
 
     The median here is somewhat close to the majority of incomes, while the
     mean is far from anybody's income.
-
-        [20 000,
-        40 000,
-        60 000,
-        9 999 999]
-    
-    The median would be around 50 000, which is close to what the majority of
-    respondents make.  The average would be in the millions, which is far from
-    what any of the respondents make.
     
     This implementation makes a temporary list of all numbers in memory.
     """
@@ -77,42 +71,20 @@ def median(r):
     high = s[center+1]
     return mean([low, high])
 
-def standard_deviation(r):
-    """Standard deviation, `from the Python Cookbook
-    <http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/442412>`_
+def standard_deviation(r, sample=True):
+    """Standard deviation. 
+    
+    `From the Python Cookbook
+    <http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/442412>`_.
+    Population mode contributed by Lorenzo Catucci.
 
     Standard deviation shows the variability within a sequence of numbers.
-    A small standard deviation shows the numbers are close to the same.  A
+    A small standard deviation means the numbers are close to each other.  A
     large standard deviation shows they are widely different.  In fact it
     shows how far the numbers tend to deviate from the average.  This can be
     used to detect whether the average has been skewed by a few extremely high
     or extremely low values.
 
-    The following examples are taken from Wikipedia.
-    http://en.wikipedia.org/wiki/Standard_deviation
-
-        >>> standard_deviation([0, 0, 14, 14])
-        8.0829037686547611
-        >>> standard_deviation([0, 6, 8, 14])
-        5.7735026918962582
-        >>> standard_deviation([6, 6, 8, 8])
-        1.1547005383792515
-
-    (Wikipedia reports 7, 5, and 1 respectively. Some of the difference is
-    due to rounding, but the rest may be a bug?)
-    
-    .. code-block:: pycon
-    
-        # Fictitious average monthly temperatures in Southern California.
-        #                       Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
-        >>> standard_deviation([70, 70, 70, 75, 80, 85, 90, 95, 90, 80, 75, 70]) 
-        9.0033663737851999
-
-        # Fictitious average mothly temperatures in Montana.
-        #                       Jan  Feb  Mar Apr May Jun Jul  Aug Sep Oct Nov Dec
-        >>> standard_deviation([-32, -10, 20, 30, 60, 90, 100, 80, 60, 30, 10, -32])
-        45.137836040557403
-
     Most natural and random phenomena follow the normal distribution (aka the
     bell curve), which says that most values are close to average but a few are
     extreme.  E.g., most people are close to 5'9" tall but a few are very tall
@@ -125,20 +97,59 @@ def standard_deviation(r):
     stdevs below get a "D" (poor), and those below 2 stdevs... we won't talk
     about them.
 
-    a large standard
-    i.e., how far they deviate from the average.
-    If all numbers are the same, the standard deviation is zero.  If the
-    numbers are widely different from average, no matter whether above or
-    below, the standard deviation will be high.  Most natural distributions
-    follow the bell curve and have a standard deviation of 1.
+    By default the helper computes the unbiased estimate
+    for the population standard deviation, by applying an unbiasing
+    factor of sqrt(N/(N-1)).
+
+    If you'd rather have the function compute the population standard
+    deviation, pass ``sample=False``.
+
+    The following examples are taken from Wikipedia.
+    http://en.wikipedia.org/wiki/Standard_deviation
+
+        >>> standard_deviation([0, 0, 14, 14]) # doctest: +ELLIPSIS
+        8.082903768654761...
+        >>> standard_deviation([0, 6, 8, 14]) # doctest: +ELLIPSIS
+        5.773502691896258...
+        >>> standard_deviation([6, 6, 8, 8])
+        1.1547005383792515
+        >>> standard_deviation([0, 0, 14, 14], sample=False)
+        7.0
+        >>> standard_deviation([0, 6, 8, 14], sample=False)
+        5.0
+        >>> standard_deviation([6, 6, 8, 8], sample=False)
+        1.0
+
+    (The results reported in Wikipedia are those expected for whole
+    population statistics and therefore are equal to the ones we get
+    by setting ``sample=False`` in the later tests.)
+    
+    .. code-block:: pycon
+    
+        # Fictitious average monthly temperatures in Southern California.
+        #                       Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
+        >>> standard_deviation([70, 70, 70, 75, 80, 85, 90, 95, 90, 80, 75, 70]) # doctest: +ELLIPSIS
+        9.003366373785...
+        >>> standard_deviation([70, 70, 70, 75, 80, 85, 90, 95, 90, 80, 75, 70], sample=False) # doctest: +ELLIPSIS
+        8.620067027323...
+
+        # Fictitious average monthly temperatures in Montana.
+        #                       Jan  Feb  Mar Apr May Jun Jul  Aug Sep Oct Nov Dec
+        >>> standard_deviation([-32, -10, 20, 30, 60, 90, 100, 80, 60, 30, 10, -32]) # doctest: +ELLIPSIS
+        45.1378360405574...
+        >>> standard_deviation([-32, -10, 20, 30, 60, 90, 100, 80, 60, 30, 10, -32], sample=False) # doctest: +ELLIPSIS
+        43.2161878106906...
     """
     avg = average(r)
     sdsq = sum([(i - avg) ** 2 for i in r])
-    return (sdsq / (len(r) - 1 or 1)) ** 0.5
-
+    if sample:
+        normal_denom=len(r) - 1 or 1
+    else:
+        normal_denom=len(r)
+    return (sdsq / normal_denom) ** 0.5
 
 class SimpleStats(object):
-    """Calculate a few simple stats on data.
+    """Calculate a few simple statistics on data.
     
     This class calculates the minimum, maximum, and count of all the values
     given to it.  The values are not saved in the object.  Usage::
@@ -168,15 +179,16 @@ class SimpleStats(object):
     >>> stats2.max
     'foo'
 
-    If the ``numeric`` constructor arg is true, only ``int``, ``long``, and 
-    ``float`` values will be accepted.  This flag is intended to enable
-    additional numeric statistics, although none are currently implemented.
-
     ``.min`` and ``.max`` are ``None`` until the first data value is
     registered.
 
     Subclasses can override ``._init_stats`` and ``._update_stats`` to add
-    additional statistics.
+    additional statistics. 
+    
+    The constructor accepts one optional argument, ``numeric``. If true, the
+    instance accepts only values that are ``int``, ``long``, or ``float``.
+    The default is false, which accepts any value. This is meant for instances
+    or subclasses that don't want non-numeric values.
     """
     __version__ = 1
 
@@ -233,8 +245,8 @@ class Stats(SimpleStats):
     >>> stats.count
     3
     >>> stats.finish()
-    >>> stats.mean
-    8.3333333333333339
+    >>> stats.mean # doctest: +ELLIPSIS
+    8.33333333333333...
     >>> stats.median
     10
     >>> stats.standard_deviation
@@ -255,8 +267,8 @@ class Stats(SimpleStats):
     ``.finish()`` and then call ``.finish()`` again.  This recalculates the
     stats over the entire data set.
 
-    The ``SimpleStats`` hook methods are available for subclasses, and 
-    additionally the ``._finish_stats`` method.
+    In addition to the hook methods provided by ``SimpleStats``, subclasses
+    can override ``._finish-stats`` to provide additional statistics.
     """
     __version__ = 1
 
@@ -270,6 +282,7 @@ class Stats(SimpleStats):
         self._init_stats()
 
     def __call__(self, value):
+        """Add a data value."""
         if self.count == 0:
             self.min = self.max = value
         else:
@@ -281,6 +294,11 @@ class Stats(SimpleStats):
         self.set.add(value)
 
     def finish(self):
+        """Finish calculations. (Call after adding all data values.)
+        
+        Call this after adding all data values, or the results will be
+        incomplete.
+        """
         self.mean = mean(self.list)
         self.median = median(self.list)
         self.standard_deviation = standard_deviation(self.list)
@@ -291,6 +309,7 @@ class Stats(SimpleStats):
         """Finish the subclass statistics now that all data are known."""
         pass
 
+#### Number formatting ####
 
 def format_number(n, thousands=",", decimal="."):
     """Format a number with a thousands separator and decimal delimeter.
@@ -325,6 +344,101 @@ def format_number(n, thousands=",", decimal="."):
         parts[0])
     return decimal.join(parts)
 
+def format_data_size(size, unit, precision=1, binary=False, full_name=False):
+    """Format a number using SI units (kilo, mega, etc.).
+
+    ``size``: The number as a float or int.
+
+    ``unit``: The unit name in plural form. Examples: "bytes", "B".
+
+    ``precision``: How many digits to the right of the decimal point. Default
+    is 1.  0 suppresses the decimal point.
+
+    ``binary``: If false, use base-10 decimal prefixes (kilo = K = 1000).  
+    If true, use base-2 binary prefixes (kibi = Ki = 1024).  
+
+    ``full_name``: If false (default), use the prefix abbreviation ("k" or
+    "Ki").  If true, use the full prefix ("kilo" or "kibi"). If false,
+    use abbreviation ("k" or "Ki").
+
+    Examples:
+
+    >>> format_data_size(1024, "B")
+    '1.0 kB'
+    >>> format_data_size(1024, "B", 2)
+    '1.02 kB'
+    >>> format_data_size(1024, "B", 2, binary=True)
+    '1.00 KiB'
+    >>> format_data_size(54000, "Wh", 0)
+    '54 kWh'
+    >>> format_data_size(85000, "m/h", 0)
+    '85 km/h'
+    >>> format_data_size(85000, "m/h", 0).replace("km/h", "klicks")
+    '85 klicks'
+    """
+    # Contributed by Wojciech Malinowski
+    if full_name is None:
+        full_name = len(unit) > 1
+        
+    if not binary:
+        base = 1000
+        if full_name:
+            multiples = ('', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa', 'zetta', 'yotta')
+        else:
+            multiples = ('', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
+    else:
+        base = 1024
+        if full_name:
+            multiples = ('', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi', 'zebi', 'yobi')
+        else:
+            multiples = ('', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi')
+            
+    if size <= 0:
+        m = 0
+    else:
+        m = int(math.log(size) / math.log(base))
+    if m > 8:
+        m = 8
+
+    if m == 0:
+        precision = '%.0f'
+    else:
+        precision = '%%.%df' % precision
+        
+    size = precision % (size / math.pow(base, m))
+
+    return '%s %s%s' % (size.strip(), multiples[m], unit)
+
+def format_byte_size(size, precision=1, binary=False, full_name=False):
+    """Same as ``format_data_size`` but specifically for bytes.
+    
+    Examples:
+
+    >>> format_byte_size(2048)
+    '2.0 kB'
+    >>> format_byte_size(2048, full_name=True)
+    '2.0 kilobytes'
+    """
+    if full_name:
+        return format_data_size(size, "bytes", precision, binary, True)
+    else:
+        return format_data_size(size, "B", precision, binary, False)
+
+def format_bit_size(size, precision=1, binary=False, full_name=False):
+    """Same as ``format_data_size`` but specifically for bytes.
+
+    Examples:
+
+    >>> format_bit_size(2048)
+    '2.0 kb'
+    >>> format_bit_size(2048, full_name=True)
+    '2.0 kilobits'
+    """
+    if full_name:
+        return format_data_size(size, "bits", precision, binary, True)
+    else:
+        return format_data_size(size, "b", precision, binary, False)
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
